/* plugin.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from plugin.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static gchar** games_plugin_real_get_mime_types (GamesPlugin* self,
                                          gint* result_length1);
static GamesPlatform** games_plugin_real_get_platforms (GamesPlugin* self,
                                                 gint* result_length1);
static GamesUriSource** games_plugin_real_get_uri_sources (GamesPlugin* self,
                                                    gint* result_length1);
static GamesUriGameFactory** games_plugin_real_get_uri_game_factories (GamesPlugin* self,
                                                                gint* result_length1);
static GamesRunnerFactory** games_plugin_real_get_runner_factories (GamesPlugin* self,
                                                             gint* result_length1);
static GType games_plugin_get_type_once (void);

static gchar**
games_plugin_real_get_mime_types (GamesPlugin* self,
                                  gint* result_length1)
{
	gchar** _tmp0_;
	gchar** _tmp1_;
	gint _tmp1__length1;
	gchar** result;
	_tmp0_ = g_new0 (gchar*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

gchar**
games_plugin_get_mime_types (GamesPlugin* self,
                             gint* result_length1)
{
	GamesPluginIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_PLUGIN_GET_INTERFACE (self);
	if (_iface_->get_mime_types) {
		return _iface_->get_mime_types (self, result_length1);
	}
	return NULL;
}

static GamesPlatform**
games_plugin_real_get_platforms (GamesPlugin* self,
                                 gint* result_length1)
{
	GamesPlatform** _tmp0_;
	GamesPlatform** _tmp1_;
	gint _tmp1__length1;
	GamesPlatform** result;
	_tmp0_ = g_new0 (GamesPlatform*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

GamesPlatform**
games_plugin_get_platforms (GamesPlugin* self,
                            gint* result_length1)
{
	GamesPluginIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_PLUGIN_GET_INTERFACE (self);
	if (_iface_->get_platforms) {
		return _iface_->get_platforms (self, result_length1);
	}
	return NULL;
}

static GamesUriSource**
games_plugin_real_get_uri_sources (GamesPlugin* self,
                                   gint* result_length1)
{
	GamesUriSource** _tmp0_;
	GamesUriSource** _tmp1_;
	gint _tmp1__length1;
	GamesUriSource** result;
	_tmp0_ = g_new0 (GamesUriSource*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

GamesUriSource**
games_plugin_get_uri_sources (GamesPlugin* self,
                              gint* result_length1)
{
	GamesPluginIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_PLUGIN_GET_INTERFACE (self);
	if (_iface_->get_uri_sources) {
		return _iface_->get_uri_sources (self, result_length1);
	}
	return NULL;
}

static GamesUriGameFactory**
games_plugin_real_get_uri_game_factories (GamesPlugin* self,
                                          gint* result_length1)
{
	GamesUriGameFactory** _tmp0_;
	GamesUriGameFactory** _tmp1_;
	gint _tmp1__length1;
	GamesUriGameFactory** result;
	_tmp0_ = g_new0 (GamesUriGameFactory*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

GamesUriGameFactory**
games_plugin_get_uri_game_factories (GamesPlugin* self,
                                     gint* result_length1)
{
	GamesPluginIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_PLUGIN_GET_INTERFACE (self);
	if (_iface_->get_uri_game_factories) {
		return _iface_->get_uri_game_factories (self, result_length1);
	}
	return NULL;
}

static GamesRunnerFactory**
games_plugin_real_get_runner_factories (GamesPlugin* self,
                                        gint* result_length1)
{
	GamesRunnerFactory** _tmp0_;
	GamesRunnerFactory** _tmp1_;
	gint _tmp1__length1;
	GamesRunnerFactory** result;
	_tmp0_ = g_new0 (GamesRunnerFactory*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

GamesRunnerFactory**
games_plugin_get_runner_factories (GamesPlugin* self,
                                   gint* result_length1)
{
	GamesPluginIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_PLUGIN_GET_INTERFACE (self);
	if (_iface_->get_runner_factories) {
		return _iface_->get_runner_factories (self, result_length1);
	}
	return NULL;
}

static void
games_plugin_default_init (GamesPluginIface * iface,
                           gpointer iface_data)
{
	iface->get_mime_types = games_plugin_real_get_mime_types;
	iface->get_platforms = games_plugin_real_get_platforms;
	iface->get_uri_sources = games_plugin_real_get_uri_sources;
	iface->get_uri_game_factories = games_plugin_real_get_uri_game_factories;
	iface->get_runner_factories = games_plugin_real_get_runner_factories;
}

static GType
games_plugin_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesPluginIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_plugin_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType games_plugin_type_id;
	games_plugin_type_id = g_type_register_static (G_TYPE_INTERFACE, "GamesPlugin", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (games_plugin_type_id, G_TYPE_OBJECT);
	return games_plugin_type_id;
}

GType
games_plugin_get_type (void)
{
	static volatile gsize games_plugin_type_id__once = 0;
	if (g_once_init_enter (&games_plugin_type_id__once)) {
		GType games_plugin_type_id;
		games_plugin_type_id = games_plugin_get_type_once ();
		g_once_init_leave (&games_plugin_type_id__once, games_plugin_type_id);
	}
	return games_plugin_type_id__once;
}

