/* nintendo-3ds-layout.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from nintendo-3ds-layout.vala, do not modify */

#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "games-nintendo-3ds-plugin.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

VALA_EXTERN gchar* games_nintendo3_ds_layout_get_option_value (GamesScreenLayout layout);

gchar*
games_nintendo3_ds_layout_get_option_value (GamesScreenLayout layout)
{
	gchar* result;
	switch (layout) {
		case GAMES_SCREEN_LAYOUT_TOP_BOTTOM:
		{
			gchar* _tmp0_;
			_tmp0_ = g_strdup ("Default Top-Bottom Screen");
			result = _tmp0_;
			return result;
		}
		case GAMES_SCREEN_LAYOUT_LEFT_RIGHT:
		case GAMES_SCREEN_LAYOUT_RIGHT_LEFT:
		{
			gchar* _tmp1_;
			_tmp1_ = g_strdup ("Side by Side");
			result = _tmp1_;
			return result;
		}
		case GAMES_SCREEN_LAYOUT_QUICK_SWITCH:
		{
			gchar* _tmp2_;
			_tmp2_ = g_strdup ("Single Screen Only");
			result = _tmp2_;
			return result;
		}
		default:
		{
			g_assert_not_reached ();
		}
	}
}

