/* nintendo-ds-platform.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from nintendo-ds-platform.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "games-nintendo-ds-plugin.h"
#include "gnome-games.h"
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	GAMES_NINTENDO_DS_PLATFORM_0_PROPERTY,
	GAMES_NINTENDO_DS_PLATFORM_NUM_PROPERTIES
};
static GParamSpec* games_nintendo_ds_platform_properties[GAMES_NINTENDO_DS_PLATFORM_NUM_PROPERTIES];

static gpointer games_nintendo_ds_platform_parent_class = NULL;
static GType games_nintendo_ds_platform_type_id = 0;

static GType games_nintendo_ds_platform_real_get_snapshot_type (GamesRetroPlatform* base);

GamesNintendoDsPlatform*
games_nintendo_ds_platform_construct (GType object_type,
                                      const gchar* id,
                                      const gchar* name,
                                      gchar** mime_types,
                                      gint mime_types_length1,
                                      const gchar* prefix)
{
	GamesNintendoDsPlatform * self = NULL;
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (prefix != NULL, NULL);
	self = (GamesNintendoDsPlatform*) games_retro_platform_construct (object_type, id, name, mime_types, (gint) mime_types_length1, prefix);
	return self;
}

GamesNintendoDsPlatform*
games_nintendo_ds_platform_new (const gchar* id,
                                const gchar* name,
                                gchar** mime_types,
                                gint mime_types_length1,
                                const gchar* prefix)
{
	return games_nintendo_ds_platform_construct (GAMES_TYPE_NINTENDO_DS_PLATFORM, id, name, mime_types, mime_types_length1, prefix);
}

static GType
games_nintendo_ds_platform_real_get_snapshot_type (GamesRetroPlatform* base)
{
	GamesNintendoDsPlatform * self;
	GType result;
	self = (GamesNintendoDsPlatform*) base;
	result = GAMES_TYPE_NINTENDO_DS_SNAPSHOT;
	return result;
}

static void
games_nintendo_ds_platform_class_init (GamesNintendoDsPlatformClass * klass,
                                       gpointer klass_data)
{
	games_nintendo_ds_platform_parent_class = g_type_class_peek_parent (klass);
	((GamesRetroPlatformClass *) klass)->get_snapshot_type = (GType (*) (GamesRetroPlatform*)) games_nintendo_ds_platform_real_get_snapshot_type;
}

static void
games_nintendo_ds_platform_instance_init (GamesNintendoDsPlatform * self,
                                          gpointer klass)
{
}

GType
games_nintendo_ds_platform_get_type (void)
{
	return games_nintendo_ds_platform_type_id;
}

GType
games_nintendo_ds_platform_register_type (GTypeModule * module)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesNintendoDsPlatformClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_nintendo_ds_platform_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GamesNintendoDsPlatform), 0, (GInstanceInitFunc) games_nintendo_ds_platform_instance_init, NULL };
	games_nintendo_ds_platform_type_id = g_type_module_register_type (module, GAMES_TYPE_RETRO_PLATFORM, "GamesNintendoDsPlatform", &g_define_type_info, 0);
	return games_nintendo_ds_platform_type_id;
}

