/* tracker-error.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from tracker-error.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

GQuark
tracker_error_quark (void)
{
	return g_quark_from_static_string ("tracker-error-quark");
}

static GType
tracker_error_get_type_once (void)
{
	static const GEnumValue values[] = {{TRACKER_ERROR_FILE_NOT_FOUND, "TRACKER_ERROR_FILE_NOT_FOUND", "file-not-found"}, {0, NULL, NULL}};
	GType tracker_error_type_id;
	tracker_error_type_id = g_enum_register_static ("TrackerError", values);
	return tracker_error_type_id;
}

GType
tracker_error_get_type (void)
{
	static volatile gsize tracker_error_type_id__once = 0;
	if (g_once_init_enter (&tracker_error_type_id__once)) {
		GType tracker_error_type_id;
		tracker_error_type_id = tracker_error_get_type_once ();
		g_once_init_leave (&tracker_error_type_id__once, tracker_error_type_id);
	}
	return tracker_error_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

