/* game-uri-adapter.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from game-uri-adapter.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType games_game_uri_adapter_get_type_once (void);

GamesGame*
games_game_uri_adapter_game_for_uri (GamesGameUriAdapter* self,
                                     GamesUri* uri,
                                     GError** error)
{
	GamesGameUriAdapterIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_GAME_URI_ADAPTER_GET_INTERFACE (self);
	if (_iface_->game_for_uri) {
		return _iface_->game_for_uri (self, uri, error);
	}
	return NULL;
}

static void
games_game_uri_adapter_default_init (GamesGameUriAdapterIface * iface,
                                     gpointer iface_data)
{
}

static GType
games_game_uri_adapter_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesGameUriAdapterIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_game_uri_adapter_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType games_game_uri_adapter_type_id;
	games_game_uri_adapter_type_id = g_type_register_static (G_TYPE_INTERFACE, "GamesGameUriAdapter", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (games_game_uri_adapter_type_id, G_TYPE_OBJECT);
	return games_game_uri_adapter_type_id;
}

GType
games_game_uri_adapter_get_type (void)
{
	static volatile gsize games_game_uri_adapter_type_id__once = 0;
	if (g_once_init_enter (&games_game_uri_adapter_type_id__once)) {
		GType games_game_uri_adapter_type_id;
		games_game_uri_adapter_type_id = games_game_uri_adapter_get_type_once ();
		g_once_init_leave (&games_game_uri_adapter_type_id__once, games_game_uri_adapter_type_id);
	}
	return games_game_uri_adapter_type_id__once;
}

