/* input-mode.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from input-mode.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType
games_input_mode_get_type_once (void)
{
	static const GEnumValue values[] = {{GAMES_INPUT_MODE_NONE, "GAMES_INPUT_MODE_NONE", "none"}, {GAMES_INPUT_MODE_KEYBOARD, "GAMES_INPUT_MODE_KEYBOARD", "keyboard"}, {GAMES_INPUT_MODE_GAMEPAD, "GAMES_INPUT_MODE_GAMEPAD", "gamepad"}, {0, NULL, NULL}};
	GType games_input_mode_type_id;
	games_input_mode_type_id = g_enum_register_static ("GamesInputMode", values);
	return games_input_mode_type_id;
}

GType
games_input_mode_get_type (void)
{
	static volatile gsize games_input_mode_type_id__once = 0;
	if (g_once_init_enter (&games_input_mode_type_id__once)) {
		GType games_input_mode_type_id;
		games_input_mode_type_id = games_input_mode_get_type_once ();
		g_once_init_leave (&games_input_mode_type_id__once, games_input_mode_type_id);
	}
	return games_input_mode_type_id__once;
}

