/* games-playstation-plugin.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __PLUGINS_PLAYSTATION_SRC_GAMES_PLAYSTATION_PLUGIN_H__
#define __PLUGINS_PLAYSTATION_SRC_GAMES_PLAYSTATION_PLUGIN_H__

#include <glib-object.h>
#include "gnome-games.h"
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GAMES_TYPE_PLAY_STATION_GAME_FACTORY (games_play_station_game_factory_get_type ())
#define GAMES_PLAY_STATION_GAME_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_PLAY_STATION_GAME_FACTORY, GamesPlayStationGameFactory))
#define GAMES_PLAY_STATION_GAME_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_PLAY_STATION_GAME_FACTORY, GamesPlayStationGameFactoryClass))
#define GAMES_IS_PLAY_STATION_GAME_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_PLAY_STATION_GAME_FACTORY))
#define GAMES_IS_PLAY_STATION_GAME_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_PLAY_STATION_GAME_FACTORY))
#define GAMES_PLAY_STATION_GAME_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_PLAY_STATION_GAME_FACTORY, GamesPlayStationGameFactoryClass))

typedef struct _GamesPlayStationGameFactory GamesPlayStationGameFactory;
typedef struct _GamesPlayStationGameFactoryClass GamesPlayStationGameFactoryClass;
typedef struct _GamesPlayStationGameFactoryPrivate GamesPlayStationGameFactoryPrivate;

struct _GamesPlayStationGameFactory {
	GObject parent_instance;
	GamesPlayStationGameFactoryPrivate * priv;
};

struct _GamesPlayStationGameFactoryClass {
	GObjectClass parent_class;
};

GType games_play_station_game_factory_register_type (GTypeModule * module);
VALA_EXTERN GType games_play_station_game_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesPlayStationGameFactory, g_object_unref)
VALA_EXTERN GamesPlayStationGameFactory* games_play_station_game_factory_new (GamesRetroPlatform* platform);
VALA_EXTERN GamesPlayStationGameFactory* games_play_station_game_factory_construct (GType object_type,
                                                                        GamesRetroPlatform* platform);
VALA_EXTERN GType register_games_plugin (GTypeModule* module);

G_END_DECLS

#endif
