/* dummy-add-collection.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from dummy-add-collection.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "gnome-games.h"
#include <glib/gi18n-lib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GAMES_TYPE_COLLECTION (games_collection_get_type ())
#define GAMES_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_COLLECTION, GamesCollection))
#define GAMES_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_COLLECTION))
#define GAMES_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GAMES_TYPE_COLLECTION, GamesCollectionIface))

typedef struct _GamesCollection GamesCollection;
typedef struct _GamesCollectionIface GamesCollectionIface;

#define GAMES_TYPE_GAME_MODEL (games_game_model_get_type ())
#define GAMES_GAME_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_GAME_MODEL, GamesGameModel))
#define GAMES_GAME_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_GAME_MODEL, GamesGameModelClass))
#define GAMES_IS_GAME_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_GAME_MODEL))
#define GAMES_IS_GAME_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_GAME_MODEL))
#define GAMES_GAME_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_GAME_MODEL, GamesGameModelClass))

typedef struct _GamesGameModel GamesGameModel;
typedef struct _GamesGameModelClass GamesGameModelClass;
typedef enum  {
	GAMES_COLLECTION_TYPE_AUTO,
	GAMES_COLLECTION_TYPE_USER,
	GAMES_COLLECTION_TYPE_PLACEHOLDER
} GamesCollectionType;

#define GAMES_TYPE_COLLECTION_TYPE (games_collection_type_get_type ())

#define GAMES_TYPE_DUMMY_ADD_COLLECTION (games_dummy_add_collection_get_type ())
#define GAMES_DUMMY_ADD_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_DUMMY_ADD_COLLECTION, GamesDummyAddCollection))
#define GAMES_DUMMY_ADD_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_DUMMY_ADD_COLLECTION, GamesDummyAddCollectionClass))
#define GAMES_IS_DUMMY_ADD_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_DUMMY_ADD_COLLECTION))
#define GAMES_IS_DUMMY_ADD_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_DUMMY_ADD_COLLECTION))
#define GAMES_DUMMY_ADD_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_DUMMY_ADD_COLLECTION, GamesDummyAddCollectionClass))

typedef struct _GamesDummyAddCollection GamesDummyAddCollection;
typedef struct _GamesDummyAddCollectionClass GamesDummyAddCollectionClass;
typedef struct _GamesDummyAddCollectionPrivate GamesDummyAddCollectionPrivate;
enum  {
	GAMES_DUMMY_ADD_COLLECTION_0_PROPERTY,
	GAMES_DUMMY_ADD_COLLECTION_IS_EMPTY_PROPERTY,
	GAMES_DUMMY_ADD_COLLECTION_TITLE_PROPERTY,
	GAMES_DUMMY_ADD_COLLECTION_NUM_PROPERTIES
};
static GParamSpec* games_dummy_add_collection_properties[GAMES_DUMMY_ADD_COLLECTION_NUM_PROPERTIES];

struct _GamesCollectionIface {
	GTypeInterface parent_iface;
	void (*load) (GamesCollection* self);
	gchar* (*get_id) (GamesCollection* self);
	gboolean (*get_hide_stars) (GamesCollection* self);
	GamesGameModel* (*get_game_model) (GamesCollection* self);
	GamesCollectionType (*get_collection_type) (GamesCollection* self);
	void (*add_games) (GamesCollection* self, GamesGame** games, gint games_length1);
	void (*remove_games) (GamesCollection* self, GamesGame** games, gint games_length1);
	void (*on_game_added) (GamesCollection* self, GamesGame* game);
	void (*on_game_removed) (GamesCollection* self, GamesGame* game);
	void (*on_game_replaced) (GamesCollection* self, GamesGame* game, GamesGame* prev_game);
	gboolean (*get_is_empty) (GamesCollection* self);
	const gchar* (*get_title) (GamesCollection* self);
};

struct _GamesDummyAddCollection {
	GObject parent_instance;
	GamesDummyAddCollectionPrivate * priv;
};

struct _GamesDummyAddCollectionClass {
	GObjectClass parent_class;
};

static gpointer games_dummy_add_collection_parent_class = NULL;
static GamesCollectionIface * games_dummy_add_collection_games_collection_parent_iface = NULL;

VALA_EXTERN GType games_game_model_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType games_collection_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType games_collection_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType games_dummy_add_collection_get_type (void) G_GNUC_CONST ;
static gchar* games_dummy_add_collection_real_get_id (GamesCollection* base);
static GamesGameModel* games_dummy_add_collection_real_get_game_model (GamesCollection* base);
VALA_EXTERN GamesGameModel* games_game_model_new (void);
VALA_EXTERN GamesGameModel* games_game_model_construct (GType object_type);
static gboolean games_dummy_add_collection_real_get_hide_stars (GamesCollection* base);
static GamesCollectionType games_dummy_add_collection_real_get_collection_type (GamesCollection* base);
static void games_dummy_add_collection_real_load (GamesCollection* base);
static void games_dummy_add_collection_real_add_games (GamesCollection* base,
                                                GamesGame** games,
                                                gint games_length1);
static void games_dummy_add_collection_real_remove_games (GamesCollection* base,
                                                   GamesGame** games,
                                                   gint games_length1);
static void games_dummy_add_collection_real_on_game_added (GamesCollection* base,
                                                    GamesGame* game);
static void games_dummy_add_collection_real_on_game_removed (GamesCollection* base,
                                                      GamesGame* game);
static void games_dummy_add_collection_real_on_game_replaced (GamesCollection* base,
                                                       GamesGame* game,
                                                       GamesGame* prev_game);
VALA_EXTERN GamesDummyAddCollection* games_dummy_add_collection_new (void);
VALA_EXTERN GamesDummyAddCollection* games_dummy_add_collection_construct (GType object_type);
VALA_EXTERN gchar* games_collection_get_id (GamesCollection* self);
VALA_EXTERN GamesGameModel* games_collection_get_game_model (GamesCollection* self);
VALA_EXTERN gboolean games_collection_get_hide_stars (GamesCollection* self);
VALA_EXTERN GamesCollectionType games_collection_get_collection_type (GamesCollection* self);
VALA_EXTERN void games_collection_load (GamesCollection* self);
VALA_EXTERN void games_collection_add_games (GamesCollection* self,
                                 GamesGame** games,
                                 gint games_length1);
VALA_EXTERN void games_collection_remove_games (GamesCollection* self,
                                    GamesGame** games,
                                    gint games_length1);
VALA_EXTERN void games_collection_on_game_added (GamesCollection* self,
                                     GamesGame* game);
VALA_EXTERN void games_collection_on_game_removed (GamesCollection* self,
                                       GamesGame* game);
VALA_EXTERN void games_collection_on_game_replaced (GamesCollection* self,
                                        GamesGame* game,
                                        GamesGame* prev_game);
VALA_EXTERN gboolean games_collection_get_is_empty (GamesCollection* self);
VALA_EXTERN const gchar* games_collection_get_title (GamesCollection* self);
static GType games_dummy_add_collection_get_type_once (void);
static void _vala_games_dummy_add_collection_get_property (GObject * object,
                                                    guint property_id,
                                                    GValue * value,
                                                    GParamSpec * pspec);

static gchar*
games_dummy_add_collection_real_get_id (GamesCollection* base)
{
	GamesDummyAddCollection * self;
	gchar* _tmp0_;
	gchar* result;
	self = (GamesDummyAddCollection*) base;
	_tmp0_ = g_strdup ("Add Collection");
	result = _tmp0_;
	return result;
}

static GamesGameModel*
games_dummy_add_collection_real_get_game_model (GamesCollection* base)
{
	GamesDummyAddCollection * self;
	GamesGameModel* _tmp0_;
	GamesGameModel* result;
	self = (GamesDummyAddCollection*) base;
	_tmp0_ = games_game_model_new ();
	result = _tmp0_;
	return result;
}

static gboolean
games_dummy_add_collection_real_get_hide_stars (GamesCollection* base)
{
	GamesDummyAddCollection * self;
	gboolean result;
	self = (GamesDummyAddCollection*) base;
	result = TRUE;
	return result;
}

static GamesCollectionType
games_dummy_add_collection_real_get_collection_type (GamesCollection* base)
{
	GamesDummyAddCollection * self;
	GamesCollectionType result;
	self = (GamesDummyAddCollection*) base;
	result = GAMES_COLLECTION_TYPE_PLACEHOLDER;
	return result;
}

static void
games_dummy_add_collection_real_load (GamesCollection* base)
{
	GamesDummyAddCollection * self;
	self = (GamesDummyAddCollection*) base;
}

static void
games_dummy_add_collection_real_add_games (GamesCollection* base,
                                           GamesGame** games,
                                           gint games_length1)
{
	GamesDummyAddCollection * self;
	self = (GamesDummyAddCollection*) base;
}

static void
games_dummy_add_collection_real_remove_games (GamesCollection* base,
                                              GamesGame** games,
                                              gint games_length1)
{
	GamesDummyAddCollection * self;
	self = (GamesDummyAddCollection*) base;
}

static void
games_dummy_add_collection_real_on_game_added (GamesCollection* base,
                                               GamesGame* game)
{
	GamesDummyAddCollection * self;
	self = (GamesDummyAddCollection*) base;
	g_return_if_fail (game != NULL);
}

static void
games_dummy_add_collection_real_on_game_removed (GamesCollection* base,
                                                 GamesGame* game)
{
	GamesDummyAddCollection * self;
	self = (GamesDummyAddCollection*) base;
	g_return_if_fail (game != NULL);
}

static void
games_dummy_add_collection_real_on_game_replaced (GamesCollection* base,
                                                  GamesGame* game,
                                                  GamesGame* prev_game)
{
	GamesDummyAddCollection * self;
	self = (GamesDummyAddCollection*) base;
	g_return_if_fail (game != NULL);
	g_return_if_fail (prev_game != NULL);
}

GamesDummyAddCollection*
games_dummy_add_collection_construct (GType object_type)
{
	GamesDummyAddCollection * self = NULL;
	self = (GamesDummyAddCollection*) g_object_new (object_type, NULL);
	return self;
}

GamesDummyAddCollection*
games_dummy_add_collection_new (void)
{
	return games_dummy_add_collection_construct (GAMES_TYPE_DUMMY_ADD_COLLECTION);
}

static gboolean
games_dummy_add_collection_real_get_is_empty (GamesCollection* base)
{
	gboolean result;
	GamesDummyAddCollection* self;
	self = (GamesDummyAddCollection*) base;
	result = TRUE;
	return result;
}

static const gchar*
games_dummy_add_collection_real_get_title (GamesCollection* base)
{
	const gchar* result;
	GamesDummyAddCollection* self;
	self = (GamesDummyAddCollection*) base;
	result = _ ("Add Collection");
	return result;
}

static void
games_dummy_add_collection_class_init (GamesDummyAddCollectionClass * klass,
                                       gpointer klass_data)
{
	games_dummy_add_collection_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = _vala_games_dummy_add_collection_get_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GAMES_DUMMY_ADD_COLLECTION_IS_EMPTY_PROPERTY, games_dummy_add_collection_properties[GAMES_DUMMY_ADD_COLLECTION_IS_EMPTY_PROPERTY] = g_param_spec_boolean ("is-empty", "is-empty", "is-empty", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GAMES_DUMMY_ADD_COLLECTION_TITLE_PROPERTY, games_dummy_add_collection_properties[GAMES_DUMMY_ADD_COLLECTION_TITLE_PROPERTY] = g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
games_dummy_add_collection_games_collection_interface_init (GamesCollectionIface * iface,
                                                            gpointer iface_data)
{
	games_dummy_add_collection_games_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_id = (gchar* (*) (GamesCollection*)) games_dummy_add_collection_real_get_id;
	iface->get_game_model = (GamesGameModel* (*) (GamesCollection*)) games_dummy_add_collection_real_get_game_model;
	iface->get_hide_stars = (gboolean (*) (GamesCollection*)) games_dummy_add_collection_real_get_hide_stars;
	iface->get_collection_type = (GamesCollectionType (*) (GamesCollection*)) games_dummy_add_collection_real_get_collection_type;
	iface->load = (void (*) (GamesCollection*)) games_dummy_add_collection_real_load;
	iface->add_games = (void (*) (GamesCollection*, GamesGame**, gint)) games_dummy_add_collection_real_add_games;
	iface->remove_games = (void (*) (GamesCollection*, GamesGame**, gint)) games_dummy_add_collection_real_remove_games;
	iface->on_game_added = (void (*) (GamesCollection*, GamesGame*)) games_dummy_add_collection_real_on_game_added;
	iface->on_game_removed = (void (*) (GamesCollection*, GamesGame*)) games_dummy_add_collection_real_on_game_removed;
	iface->on_game_replaced = (void (*) (GamesCollection*, GamesGame*, GamesGame*)) games_dummy_add_collection_real_on_game_replaced;
	iface->get_is_empty = games_dummy_add_collection_real_get_is_empty;
	iface->get_title = games_dummy_add_collection_real_get_title;
}

static void
games_dummy_add_collection_instance_init (GamesDummyAddCollection * self,
                                          gpointer klass)
{
}

static GType
games_dummy_add_collection_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesDummyAddCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_dummy_add_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GamesDummyAddCollection), 0, (GInstanceInitFunc) games_dummy_add_collection_instance_init, NULL };
	static const GInterfaceInfo games_collection_info = { (GInterfaceInitFunc) games_dummy_add_collection_games_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType games_dummy_add_collection_type_id;
	games_dummy_add_collection_type_id = g_type_register_static (G_TYPE_OBJECT, "GamesDummyAddCollection", &g_define_type_info, 0);
	g_type_add_interface_static (games_dummy_add_collection_type_id, GAMES_TYPE_COLLECTION, &games_collection_info);
	return games_dummy_add_collection_type_id;
}

GType
games_dummy_add_collection_get_type (void)
{
	static volatile gsize games_dummy_add_collection_type_id__once = 0;
	if (g_once_init_enter (&games_dummy_add_collection_type_id__once)) {
		GType games_dummy_add_collection_type_id;
		games_dummy_add_collection_type_id = games_dummy_add_collection_get_type_once ();
		g_once_init_leave (&games_dummy_add_collection_type_id__once, games_dummy_add_collection_type_id);
	}
	return games_dummy_add_collection_type_id__once;
}

static void
_vala_games_dummy_add_collection_get_property (GObject * object,
                                               guint property_id,
                                               GValue * value,
                                               GParamSpec * pspec)
{
	GamesDummyAddCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GAMES_TYPE_DUMMY_ADD_COLLECTION, GamesDummyAddCollection);
	switch (property_id) {
		case GAMES_DUMMY_ADD_COLLECTION_IS_EMPTY_PROPERTY:
		g_value_set_boolean (value, games_collection_get_is_empty ((GamesCollection*) self));
		break;
		case GAMES_DUMMY_ADD_COLLECTION_TITLE_PROPERTY:
		g_value_set_string (value, games_collection_get_title ((GamesCollection*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

