/* ui-view.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from ui-view.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>
#include <libmanette.h>
#include <string.h>
#include "gnome-games.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GAMES_TYPE_UI_VIEW (games_ui_view_get_type ())
#define GAMES_UI_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_UI_VIEW, GamesUiView))
#define GAMES_IS_UI_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_UI_VIEW))
#define GAMES_UI_VIEW_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GAMES_TYPE_UI_VIEW, GamesUiViewIface))

typedef struct _GamesUiView GamesUiView;
typedef struct _GamesUiViewIface GamesUiViewIface;

struct _GamesUiViewIface {
	GTypeInterface parent_iface;
	gboolean (*on_button_pressed) (GamesUiView* self, GdkEventButton* event);
	gboolean (*on_key_pressed) (GamesUiView* self, GdkEventKey* event);
	gboolean (*gamepad_button_press_event) (GamesUiView* self, ManetteEvent* event);
	gboolean (*gamepad_button_release_event) (GamesUiView* self, ManetteEvent* event);
	gboolean (*gamepad_absolute_axis_event) (GamesUiView* self, ManetteEvent* event);
	gboolean (*get_is_view_active) (GamesUiView* self);
	void (*set_is_view_active) (GamesUiView* self, gboolean value);
};

VALA_EXTERN GType games_ui_view_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gboolean games_ui_view_on_button_pressed (GamesUiView* self,
                                          GdkEventButton* event);
VALA_EXTERN gboolean games_ui_view_on_key_pressed (GamesUiView* self,
                                       GdkEventKey* event);
VALA_EXTERN gboolean games_ui_view_gamepad_button_press_event (GamesUiView* self,
                                                   ManetteEvent* event);
VALA_EXTERN gboolean games_ui_view_gamepad_button_release_event (GamesUiView* self,
                                                     ManetteEvent* event);
VALA_EXTERN gboolean games_ui_view_gamepad_absolute_axis_event (GamesUiView* self,
                                                    ManetteEvent* event);
VALA_EXTERN gboolean games_ui_view_get_is_view_active (GamesUiView* self);
VALA_EXTERN void games_ui_view_set_is_view_active (GamesUiView* self,
                                       gboolean value);
static GType games_ui_view_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

gboolean
games_ui_view_on_button_pressed (GamesUiView* self,
                                 GdkEventButton* event)
{
	GamesUiViewIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->on_button_pressed) {
		return _iface_->on_button_pressed (self, event);
	}
	return FALSE;
}

gboolean
games_ui_view_on_key_pressed (GamesUiView* self,
                              GdkEventKey* event)
{
	GamesUiViewIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->on_key_pressed) {
		return _iface_->on_key_pressed (self, event);
	}
	return FALSE;
}

gboolean
games_ui_view_gamepad_button_press_event (GamesUiView* self,
                                          ManetteEvent* event)
{
	GamesUiViewIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->gamepad_button_press_event) {
		return _iface_->gamepad_button_press_event (self, event);
	}
	return FALSE;
}

gboolean
games_ui_view_gamepad_button_release_event (GamesUiView* self,
                                            ManetteEvent* event)
{
	GamesUiViewIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->gamepad_button_release_event) {
		return _iface_->gamepad_button_release_event (self, event);
	}
	return FALSE;
}

gboolean
games_ui_view_gamepad_absolute_axis_event (GamesUiView* self,
                                           ManetteEvent* event)
{
	GamesUiViewIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->gamepad_absolute_axis_event) {
		return _iface_->gamepad_absolute_axis_event (self, event);
	}
	return FALSE;
}

gboolean
games_ui_view_get_is_view_active (GamesUiView* self)
{
	GamesUiViewIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->get_is_view_active) {
		return _iface_->get_is_view_active (self);
	}
	return FALSE;
}

void
games_ui_view_set_is_view_active (GamesUiView* self,
                                  gboolean value)
{
	GamesUiViewIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = GAMES_UI_VIEW_GET_INTERFACE (self);
	if (_iface_->set_is_view_active) {
		_iface_->set_is_view_active (self, value);
	}
}

static void
games_ui_view_default_init (GamesUiViewIface * iface,
                            gpointer iface_data)
{
	g_object_interface_install_property (iface, g_param_spec_boolean ("is-view-active", "is-view-active", "is-view-active", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static GType
games_ui_view_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesUiViewIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_ui_view_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType games_ui_view_type_id;
	games_ui_view_type_id = g_type_register_static (G_TYPE_INTERFACE, "GamesUiView", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (games_ui_view_type_id, gtk_widget_get_type ());
	return games_ui_view_type_id;
}

GType
games_ui_view_get_type (void)
{
	static volatile gsize games_ui_view_type_id__once = 0;
	if (g_once_init_enter (&games_ui_view_type_id__once)) {
		GType games_ui_view_type_id;
		games_ui_view_type_id = games_ui_view_get_type_once ();
		g_once_init_leave (&games_ui_view_type_id__once, games_ui_view_type_id);
	}
	return games_ui_view_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

