/* games-steam-plugin.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __PLUGINS_STEAM_SRC_GAMES_STEAM_PLUGIN_H__
#define __PLUGINS_STEAM_SRC_GAMES_STEAM_PLUGIN_H__

#include <glib-object.h>
#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GAMES_TYPE_STEAM_COVER (games_steam_cover_get_type ())
#define GAMES_STEAM_COVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_STEAM_COVER, GamesSteamCover))
#define GAMES_STEAM_COVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_STEAM_COVER, GamesSteamCoverClass))
#define GAMES_IS_STEAM_COVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_STEAM_COVER))
#define GAMES_IS_STEAM_COVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_STEAM_COVER))
#define GAMES_STEAM_COVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_STEAM_COVER, GamesSteamCoverClass))

typedef struct _GamesSteamCover GamesSteamCover;
typedef struct _GamesSteamCoverClass GamesSteamCoverClass;
typedef struct _GamesSteamCoverPrivate GamesSteamCoverPrivate;

struct _GamesSteamCover {
	GObject parent_instance;
	GamesSteamCoverPrivate * priv;
};

struct _GamesSteamCoverClass {
	GObjectClass parent_class;
};

GType games_steam_cover_register_type (GTypeModule * module);
VALA_EXTERN GType games_steam_cover_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesSteamCover, g_object_unref)
VALA_EXTERN GamesSteamCover* games_steam_cover_new (const gchar* game_id);
VALA_EXTERN GamesSteamCover* games_steam_cover_construct (GType object_type,
                                              const gchar* game_id);
VALA_EXTERN GType register_games_plugin (GTypeModule* module);

G_END_DECLS

#endif
