/* credits.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from credits.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "gnome-games.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

const gchar* GAMES_CREDITS_AUTHORS[10] = {"Abhinav Singh <theawless@gmail.com>", "Adrien Plazas <kekun.plazas@laposte.net>", "Alexander Mikhaylenko <exalm7659@gmail.com>", "Andrei Lişiţă <andreii.lisita@gmail.com>", "Megh Parikh <meghprkh@gmail.com>", "Neville Antony <nevilleantony98@gmail.com>", "Ricard Gascons <gascons1995@gmail.com>", "Saurabh Singh <saurabhsingh412@gmail.com>", "Sebastien Nicouleaud <sebn@users.noreply.github.com>", NULL};
const gchar* GAMES_CREDITS_ARTISTS[4] = {"Adrien Plazas <kekun.plazas@laposte.net>", "Jakub Steiner <jimmac@gmail.com>", "Tobias Bernard <tbernard@gnome.org>", NULL};
const gchar* GAMES_CREDITS_DOCUMENTERS[3] = {"Adrien Plazas <kekun.plazas@laposte.net>", "Laurent Pointecouteau <laurent.pointecouteau@gmail.com>", NULL};

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

