/* games-nintendo-ds-plugin.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __PLUGINS_NINTENDO_DS_SRC_GAMES_NINTENDO_DS_PLUGIN_H__
#define __PLUGINS_NINTENDO_DS_SRC_GAMES_NINTENDO_DS_PLUGIN_H__

#include <glib-object.h>
#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GAMES_TYPE_NINTENDO_DS_ICON (games_nintendo_ds_icon_get_type ())
#define GAMES_NINTENDO_DS_ICON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_NINTENDO_DS_ICON, GamesNintendoDsIcon))
#define GAMES_NINTENDO_DS_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_NINTENDO_DS_ICON, GamesNintendoDsIconClass))
#define GAMES_IS_NINTENDO_DS_ICON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_NINTENDO_DS_ICON))
#define GAMES_IS_NINTENDO_DS_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_NINTENDO_DS_ICON))
#define GAMES_NINTENDO_DS_ICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_NINTENDO_DS_ICON, GamesNintendoDsIconClass))

typedef struct _GamesNintendoDsIcon GamesNintendoDsIcon;
typedef struct _GamesNintendoDsIconClass GamesNintendoDsIconClass;
typedef struct _GamesNintendoDsIconPrivate GamesNintendoDsIconPrivate;

#define GAMES_TYPE_NINTENDO_DS_PLATFORM (games_nintendo_ds_platform_get_type ())
#define GAMES_NINTENDO_DS_PLATFORM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_NINTENDO_DS_PLATFORM, GamesNintendoDsPlatform))
#define GAMES_NINTENDO_DS_PLATFORM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_NINTENDO_DS_PLATFORM, GamesNintendoDsPlatformClass))
#define GAMES_IS_NINTENDO_DS_PLATFORM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_NINTENDO_DS_PLATFORM))
#define GAMES_IS_NINTENDO_DS_PLATFORM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_NINTENDO_DS_PLATFORM))
#define GAMES_NINTENDO_DS_PLATFORM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_NINTENDO_DS_PLATFORM, GamesNintendoDsPlatformClass))

typedef struct _GamesNintendoDsPlatform GamesNintendoDsPlatform;
typedef struct _GamesNintendoDsPlatformClass GamesNintendoDsPlatformClass;
typedef struct _GamesNintendoDsPlatformPrivate GamesNintendoDsPlatformPrivate;

#define GAMES_TYPE_NINTENDO_DS_SNAPSHOT (games_nintendo_ds_snapshot_get_type ())
#define GAMES_NINTENDO_DS_SNAPSHOT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_NINTENDO_DS_SNAPSHOT, GamesNintendoDsSnapshot))
#define GAMES_NINTENDO_DS_SNAPSHOT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_NINTENDO_DS_SNAPSHOT, GamesNintendoDsSnapshotClass))
#define GAMES_IS_NINTENDO_DS_SNAPSHOT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_NINTENDO_DS_SNAPSHOT))
#define GAMES_IS_NINTENDO_DS_SNAPSHOT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_NINTENDO_DS_SNAPSHOT))
#define GAMES_NINTENDO_DS_SNAPSHOT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_NINTENDO_DS_SNAPSHOT, GamesNintendoDsSnapshotClass))

typedef struct _GamesNintendoDsSnapshot GamesNintendoDsSnapshot;
typedef struct _GamesNintendoDsSnapshotClass GamesNintendoDsSnapshotClass;
typedef struct _GamesNintendoDsSnapshotPrivate GamesNintendoDsSnapshotPrivate;

struct _GamesNintendoDsIcon {
	GObject parent_instance;
	GamesNintendoDsIconPrivate * priv;
};

struct _GamesNintendoDsIconClass {
	GObjectClass parent_class;
};

struct _GamesNintendoDsPlatform {
	GamesRetroPlatform parent_instance;
	GamesNintendoDsPlatformPrivate * priv;
};

struct _GamesNintendoDsPlatformClass {
	GamesRetroPlatformClass parent_class;
};

struct _GamesNintendoDsSnapshot {
	GamesSnapshot parent_instance;
	GamesNintendoDsSnapshotPrivate * priv;
};

struct _GamesNintendoDsSnapshotClass {
	GamesSnapshotClass parent_class;
};

GType games_nintendo_ds_icon_register_type (GTypeModule * module);
VALA_EXTERN GType games_nintendo_ds_icon_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesNintendoDsIcon, g_object_unref)
VALA_EXTERN GamesNintendoDsIcon* games_nintendo_ds_icon_new (GamesUri* uri);
VALA_EXTERN GamesNintendoDsIcon* games_nintendo_ds_icon_construct (GType object_type,
                                                       GamesUri* uri);
GType games_nintendo_ds_platform_register_type (GTypeModule * module);
VALA_EXTERN GType games_nintendo_ds_platform_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesNintendoDsPlatform, g_object_unref)
VALA_EXTERN GamesNintendoDsPlatform* games_nintendo_ds_platform_new (const gchar* id,
                                                         const gchar* name,
                                                         gchar** mime_types,
                                                         gint mime_types_length1,
                                                         const gchar* prefix);
VALA_EXTERN GamesNintendoDsPlatform* games_nintendo_ds_platform_construct (GType object_type,
                                                               const gchar* id,
                                                               const gchar* name,
                                                               gchar** mime_types,
                                                               gint mime_types_length1,
                                                               const gchar* prefix);
VALA_EXTERN GType register_games_plugin (GTypeModule* module);
GType games_nintendo_ds_snapshot_register_type (GTypeModule * module);
VALA_EXTERN GType games_nintendo_ds_snapshot_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesNintendoDsSnapshot, g_object_unref)
VALA_EXTERN GamesNintendoDsSnapshot* games_nintendo_ds_snapshot_new (void);
VALA_EXTERN GamesNintendoDsSnapshot* games_nintendo_ds_snapshot_construct (GType object_type);
VALA_EXTERN GamesScreenLayout games_nintendo_ds_snapshot_get_screen_layout (GamesNintendoDsSnapshot* self);
VALA_EXTERN void games_nintendo_ds_snapshot_set_screen_layout (GamesNintendoDsSnapshot* self,
                                                   GamesScreenLayout value);
VALA_EXTERN gboolean games_nintendo_ds_snapshot_get_view_bottom_screen (GamesNintendoDsSnapshot* self);
VALA_EXTERN void games_nintendo_ds_snapshot_set_view_bottom_screen (GamesNintendoDsSnapshot* self,
                                                        gboolean value);

G_END_DECLS

#endif
