/* retro-simple-types.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from retro-simple-types.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "gnome-games.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GAMES_TYPE_RETRO_SIMPLE_TYPE (games_retro_simple_type_get_type ())
typedef struct _GamesRetroSimpleType GamesRetroSimpleType;

struct _GamesRetroSimpleType {
	gchar* mime_type;
	gboolean search_mime_type;
	gchar* platform;
	gchar* prefix;
};

VALA_EXTERN GType games_retro_simple_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GamesRetroSimpleType* games_retro_simple_type_dup (const GamesRetroSimpleType* self);
VALA_EXTERN void games_retro_simple_type_free (GamesRetroSimpleType* self);
VALA_EXTERN void games_retro_simple_type_copy (const GamesRetroSimpleType* self,
                                   GamesRetroSimpleType* dest);
VALA_EXTERN void games_retro_simple_type_destroy (GamesRetroSimpleType* self);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

const GamesRetroSimpleType GAMES_RETRO_SIMPLE_TYPES[23] = {{"application/vnd.nintendo.snes.rom", TRUE, "SuperNintendoEntertainmentSystem", "snes"}, {"application/x-amiga-disk-format", TRUE, "Amiga", "amiga"}, {"application/x-atari-2600-rom", TRUE, "Atari2600", "atari-2600"}, {"application/x-atari-7800-rom", TRUE, "Atari7800", "atari-7800"}, {"application/x-atari-lynx-rom", TRUE, "AtariLynx", "atari-lynx"}, {"application/x-doom-wad", TRUE, "DOOM", "doom"}, {"application/x-fds-disk", TRUE, "FamicomDiskSystem", "fds"}, {"application/x-gameboy-color-rom", TRUE, "GameBoyColor", "game-boy"}, {"application/x-gameboy-rom", TRUE, "GameBoy", "game-boy"}, {"application/x-gamegear-rom", TRUE, "GameGear", "game-gear"}, {"application/x-gba-rom", TRUE, "GameBoyAdvance", "game-boy-advance"}, {"application/x-genesis-32x-rom", TRUE, "Sega32X", "mega-drive-32x"}, {"application/x-genesis-rom", TRUE, "SegaGenesis", "mega-drive"}, {"application/x-neo-geo-pocket-color-rom", TRUE, "NeoGeoPocketColor", "neo-geo-pocket"}, {"application/x-neo-geo-pocket-rom", TRUE, "NeoGeoPocket", "neo-geo-pocket"}, {"application/x-nes-rom", TRUE, "NintendoEntertainmentSystem", "nes"}, {"application/x-pc-engine-rom", TRUE, "TurboGrafx16", "pc-engine"}, {"application/x-sega-pico-rom", TRUE, "SegaPico", "sega-pico"}, {"application/x-sg1000-rom", TRUE, "SG1000", "sg-1000"}, {"application/x-sms-rom", TRUE, "MasterSystem", "master-system"}, {"application/x-wii-wad", TRUE, "WiiWare", "wii-ware"}, {"application/x-wonderswan-rom", TRUE, "WonderSwan", "wonderswan"}, {"application/x-wonderswan-color-rom", TRUE, "WonderSwanColor", "wonderswan-color"}};

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

