/* dummy-icon.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from dummy-icon.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <gio/gio.h>
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	GAMES_DUMMY_ICON_0_PROPERTY,
	GAMES_DUMMY_ICON_NUM_PROPERTIES
};
static GParamSpec* games_dummy_icon_properties[GAMES_DUMMY_ICON_NUM_PROPERTIES];

static gpointer games_dummy_icon_parent_class = NULL;
static GamesIconIface * games_dummy_icon_games_icon_parent_iface = NULL;

static GIcon* games_dummy_icon_real_get_icon (GamesIcon* base);
static GType games_dummy_icon_get_type_once (void);

static GIcon*
games_dummy_icon_real_get_icon (GamesIcon* base)
{
	GamesDummyIcon * self;
	GIcon* result;
	self = (GamesDummyIcon*) base;
	result = NULL;
	return result;
}

GamesDummyIcon*
games_dummy_icon_construct (GType object_type)
{
	GamesDummyIcon * self = NULL;
	self = (GamesDummyIcon*) g_object_new (object_type, NULL);
	return self;
}

GamesDummyIcon*
games_dummy_icon_new (void)
{
	return games_dummy_icon_construct (GAMES_TYPE_DUMMY_ICON);
}

static void
games_dummy_icon_class_init (GamesDummyIconClass * klass,
                             gpointer klass_data)
{
	games_dummy_icon_parent_class = g_type_class_peek_parent (klass);
}

static void
games_dummy_icon_games_icon_interface_init (GamesIconIface * iface,
                                            gpointer iface_data)
{
	games_dummy_icon_games_icon_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_icon = (GIcon* (*) (GamesIcon*)) games_dummy_icon_real_get_icon;
}

static void
games_dummy_icon_instance_init (GamesDummyIcon * self,
                                gpointer klass)
{
}

static GType
games_dummy_icon_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesDummyIconClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_dummy_icon_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GamesDummyIcon), 0, (GInstanceInitFunc) games_dummy_icon_instance_init, NULL };
	static const GInterfaceInfo games_icon_info = { (GInterfaceInitFunc) games_dummy_icon_games_icon_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType games_dummy_icon_type_id;
	games_dummy_icon_type_id = g_type_register_static (G_TYPE_OBJECT, "GamesDummyIcon", &g_define_type_info, 0);
	g_type_add_interface_static (games_dummy_icon_type_id, GAMES_TYPE_ICON, &games_icon_info);
	return games_dummy_icon_type_id;
}

GType
games_dummy_icon_get_type (void)
{
	static volatile gsize games_dummy_icon_type_id__once = 0;
	if (g_once_init_enter (&games_dummy_icon_type_id__once)) {
		GType games_dummy_icon_type_id;
		games_dummy_icon_type_id = games_dummy_icon_get_type_once ();
		g_once_init_leave (&games_dummy_icon_type_id__once, games_dummy_icon_type_id);
	}
	return games_dummy_icon_type_id__once;
}

