/* games-desktop-plugin.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __PLUGINS_DESKTOP_SRC_GAMES_DESKTOP_PLUGIN_H__
#define __PLUGINS_DESKTOP_SRC_GAMES_DESKTOP_PLUGIN_H__

#include <glib-object.h>
#include "gnome-games.h"
#include <gio/gdesktopappinfo.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GAMES_TYPE_DESKTOP_ICON (games_desktop_icon_get_type ())
#define GAMES_DESKTOP_ICON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_DESKTOP_ICON, GamesDesktopIcon))
#define GAMES_DESKTOP_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_DESKTOP_ICON, GamesDesktopIconClass))
#define GAMES_IS_DESKTOP_ICON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_DESKTOP_ICON))
#define GAMES_IS_DESKTOP_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_DESKTOP_ICON))
#define GAMES_DESKTOP_ICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_DESKTOP_ICON, GamesDesktopIconClass))

typedef struct _GamesDesktopIcon GamesDesktopIcon;
typedef struct _GamesDesktopIconClass GamesDesktopIconClass;
typedef struct _GamesDesktopIconPrivate GamesDesktopIconPrivate;

#define GAMES_TYPE_DESKTOP_TITLE (games_desktop_title_get_type ())
#define GAMES_DESKTOP_TITLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_DESKTOP_TITLE, GamesDesktopTitle))
#define GAMES_DESKTOP_TITLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_DESKTOP_TITLE, GamesDesktopTitleClass))
#define GAMES_IS_DESKTOP_TITLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_DESKTOP_TITLE))
#define GAMES_IS_DESKTOP_TITLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_DESKTOP_TITLE))
#define GAMES_DESKTOP_TITLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_DESKTOP_TITLE, GamesDesktopTitleClass))

typedef struct _GamesDesktopTitle GamesDesktopTitle;
typedef struct _GamesDesktopTitleClass GamesDesktopTitleClass;
typedef struct _GamesDesktopTitlePrivate GamesDesktopTitlePrivate;

struct _GamesDesktopIcon {
	GObject parent_instance;
	GamesDesktopIconPrivate * priv;
};

struct _GamesDesktopIconClass {
	GObjectClass parent_class;
};

struct _GamesDesktopTitle {
	GObject parent_instance;
	GamesDesktopTitlePrivate * priv;
};

struct _GamesDesktopTitleClass {
	GObjectClass parent_class;
};

GType games_desktop_icon_register_type (GTypeModule * module);
VALA_EXTERN GType games_desktop_icon_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesDesktopIcon, g_object_unref)
VALA_EXTERN GamesDesktopIcon* games_desktop_icon_new (GDesktopAppInfo* app_info);
VALA_EXTERN GamesDesktopIcon* games_desktop_icon_construct (GType object_type,
                                                GDesktopAppInfo* app_info);
VALA_EXTERN GType register_games_plugin (GTypeModule* module);
GType games_desktop_title_register_type (GTypeModule * module);
VALA_EXTERN GType games_desktop_title_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GamesDesktopTitle, g_object_unref)
VALA_EXTERN GamesDesktopTitle* games_desktop_title_new (GDesktopAppInfo* app_info);
VALA_EXTERN GamesDesktopTitle* games_desktop_title_construct (GType object_type,
                                                  GDesktopAppInfo* app_info);

G_END_DECLS

#endif
