/* dummy-platform.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from dummy-platform.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include <handy.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	GAMES_DUMMY_PLATFORM_0_PROPERTY,
	GAMES_DUMMY_PLATFORM_NUM_PROPERTIES
};
static GParamSpec* games_dummy_platform_properties[GAMES_DUMMY_PLATFORM_NUM_PROPERTIES];

static gpointer games_dummy_platform_parent_class = NULL;
static GamesPlatformIface * games_dummy_platform_games_platform_parent_iface = NULL;

static gchar* games_dummy_platform_real_get_id (GamesPlatform* base);
static gchar* games_dummy_platform_real_get_name (GamesPlatform* base);
static gchar* games_dummy_platform_real_get_uid_prefix (GamesPlatform* base);
static GtkListBoxRow* games_dummy_platform_real_get_row (GamesPlatform* base);
static GType games_dummy_platform_real_get_snapshot_type (GamesPlatform* base);
static GType games_dummy_platform_get_type_once (void);

static gchar*
games_dummy_platform_real_get_id (GamesPlatform* base)
{
	GamesDummyPlatform * self;
	gchar* _tmp0_;
	gchar* result;
	self = (GamesDummyPlatform*) base;
	_tmp0_ = g_strdup ("Unknown");
	result = _tmp0_;
	return result;
}

static gchar*
games_dummy_platform_real_get_name (GamesPlatform* base)
{
	GamesDummyPlatform * self;
	gchar* _tmp0_;
	gchar* result;
	self = (GamesDummyPlatform*) base;
	_tmp0_ = g_strdup (_ ("Unknown"));
	result = _tmp0_;
	return result;
}

static gchar*
games_dummy_platform_real_get_uid_prefix (GamesPlatform* base)
{
	GamesDummyPlatform * self;
	gchar* _tmp0_;
	gchar* result;
	self = (GamesDummyPlatform*) base;
	_tmp0_ = g_strdup ("unknown");
	result = _tmp0_;
	return result;
}

static GtkListBoxRow*
games_dummy_platform_real_get_row (GamesPlatform* base)
{
	GamesDummyPlatform * self;
	HdyActionRow* generic_row = NULL;
	HdyActionRow* _tmp0_;
	GtkListBoxRow* result;
	self = (GamesDummyPlatform*) base;
	_tmp0_ = (HdyActionRow*) hdy_action_row_new ();
	g_object_ref_sink (_tmp0_);
	generic_row = _tmp0_;
	hdy_preferences_row_set_title ((HdyPreferencesRow*) generic_row, _ ("Unknown"));
	result = (GtkListBoxRow*) generic_row;
	return result;
}

static GType
games_dummy_platform_real_get_snapshot_type (GamesPlatform* base)
{
	GamesDummyPlatform * self;
	GType result;
	self = (GamesDummyPlatform*) base;
	result = GAMES_TYPE_SNAPSHOT;
	return result;
}

GamesDummyPlatform*
games_dummy_platform_construct (GType object_type)
{
	GamesDummyPlatform * self = NULL;
	self = (GamesDummyPlatform*) g_object_new (object_type, NULL);
	return self;
}

GamesDummyPlatform*
games_dummy_platform_new (void)
{
	return games_dummy_platform_construct (GAMES_TYPE_DUMMY_PLATFORM);
}

static void
games_dummy_platform_class_init (GamesDummyPlatformClass * klass,
                                 gpointer klass_data)
{
	games_dummy_platform_parent_class = g_type_class_peek_parent (klass);
}

static void
games_dummy_platform_games_platform_interface_init (GamesPlatformIface * iface,
                                                    gpointer iface_data)
{
	games_dummy_platform_games_platform_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_id = (gchar* (*) (GamesPlatform*)) games_dummy_platform_real_get_id;
	iface->get_name = (gchar* (*) (GamesPlatform*)) games_dummy_platform_real_get_name;
	iface->get_uid_prefix = (gchar* (*) (GamesPlatform*)) games_dummy_platform_real_get_uid_prefix;
	iface->get_row = (GtkListBoxRow* (*) (GamesPlatform*)) games_dummy_platform_real_get_row;
	iface->get_snapshot_type = (GType (*) (GamesPlatform*)) games_dummy_platform_real_get_snapshot_type;
}

static void
games_dummy_platform_instance_init (GamesDummyPlatform * self,
                                    gpointer klass)
{
}

static GType
games_dummy_platform_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesDummyPlatformClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_dummy_platform_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GamesDummyPlatform), 0, (GInstanceInitFunc) games_dummy_platform_instance_init, NULL };
	static const GInterfaceInfo games_platform_info = { (GInterfaceInitFunc) games_dummy_platform_games_platform_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType games_dummy_platform_type_id;
	games_dummy_platform_type_id = g_type_register_static (G_TYPE_OBJECT, "GamesDummyPlatform", &g_define_type_info, 0);
	g_type_add_interface_static (games_dummy_platform_type_id, GAMES_TYPE_PLATFORM, &games_platform_info);
	return games_dummy_platform_type_id;
}

GType
games_dummy_platform_get_type (void)
{
	static volatile gsize games_dummy_platform_type_id__once = 0;
	if (g_once_init_enter (&games_dummy_platform_type_id__once)) {
		GType games_dummy_platform_type_id;
		games_dummy_platform_type_id = games_dummy_platform_get_type_once ();
		g_once_init_leave (&games_dummy_platform_type_id__once, games_dummy_platform_type_id);
	}
	return games_dummy_platform_type_id__once;
}

