/* sega-cd-system.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from sega-cd-system.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <glib-object.h>
#include "games-sega-cd-plugin.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

typedef enum  {
	GAMES_SEGA_CD_SYSTEM_INVALID,
	GAMES_SEGA_CD_SYSTEM_SEGA_CD,
	GAMES_SEGA_CD_SYSTEM_SEGA_CD_32X
} GamesSegaCDSystem;

#define GAMES_TYPE_SEGA_CD_SYSTEM (games_sega_cd_system_get_type ())

VALA_EXTERN GType games_sega_cd_system_get_type (void) G_GNUC_CONST ;

static GType
games_sega_cd_system_get_type_once (void)
{
	static const GEnumValue values[] = {{GAMES_SEGA_CD_SYSTEM_INVALID, "GAMES_SEGA_CD_SYSTEM_INVALID", "invalid"}, {GAMES_SEGA_CD_SYSTEM_SEGA_CD, "GAMES_SEGA_CD_SYSTEM_SEGA_CD", "sega-cd"}, {GAMES_SEGA_CD_SYSTEM_SEGA_CD_32X, "GAMES_SEGA_CD_SYSTEM_SEGA_CD_32X", "sega-cd-32x"}, {0, NULL, NULL}};
	GType games_sega_cd_system_type_id;
	games_sega_cd_system_type_id = g_enum_register_static ("GamesSegaCDSystem", values);
	return games_sega_cd_system_type_id;
}

GType
games_sega_cd_system_get_type (void)
{
	static volatile gsize games_sega_cd_system_type_id__once = 0;
	if (g_once_init_enter (&games_sega_cd_system_type_id__once)) {
		GType games_sega_cd_system_type_id;
		games_sega_cd_system_type_id = games_sega_cd_system_get_type_once ();
		g_once_init_leave (&games_sega_cd_system_type_id__once, games_sega_cd_system_type_id);
	}
	return games_sega_cd_system_type_id__once;
}

