/* libretro-uri-source.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from libretro-uri-source.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "games-libretro-plugin.h"
#include "gnome-games.h"
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	GAMES_LIBRETRO_URI_SOURCE_0_PROPERTY,
	GAMES_LIBRETRO_URI_SOURCE_NUM_PROPERTIES
};
static GParamSpec* games_libretro_uri_source_properties[GAMES_LIBRETRO_URI_SOURCE_NUM_PROPERTIES];

static gpointer games_libretro_uri_source_parent_class = NULL;
static GamesUriSourceIface * games_libretro_uri_source_games_uri_source_parent_iface = NULL;
static GType games_libretro_uri_source_type_id = 0;

static GamesUriIterator* games_libretro_uri_source_real_iterator (GamesUriSource* base);

static GamesUriIterator*
games_libretro_uri_source_real_iterator (GamesUriSource* base)
{
	GamesLibretroUriSource * self;
	GamesLibretroUriIterator* _tmp0_;
	GamesUriIterator* result;
	self = (GamesLibretroUriSource*) base;
	_tmp0_ = games_libretro_uri_iterator_new ();
	result = (GamesUriIterator*) _tmp0_;
	return result;
}

GamesLibretroUriSource*
games_libretro_uri_source_construct (GType object_type)
{
	GamesLibretroUriSource * self = NULL;
	self = (GamesLibretroUriSource*) g_object_new (object_type, NULL);
	return self;
}

GamesLibretroUriSource*
games_libretro_uri_source_new (void)
{
	return games_libretro_uri_source_construct (GAMES_TYPE_LIBRETRO_URI_SOURCE);
}

static void
games_libretro_uri_source_class_init (GamesLibretroUriSourceClass * klass,
                                      gpointer klass_data)
{
	games_libretro_uri_source_parent_class = g_type_class_peek_parent (klass);
}

static void
games_libretro_uri_source_games_uri_source_interface_init (GamesUriSourceIface * iface,
                                                           gpointer iface_data)
{
	games_libretro_uri_source_games_uri_source_parent_iface = g_type_interface_peek_parent (iface);
	iface->iterator = (GamesUriIterator* (*) (GamesUriSource*)) games_libretro_uri_source_real_iterator;
}

static void
games_libretro_uri_source_instance_init (GamesLibretroUriSource * self,
                                         gpointer klass)
{
}

GType
games_libretro_uri_source_get_type (void)
{
	return games_libretro_uri_source_type_id;
}

GType
games_libretro_uri_source_register_type (GTypeModule * module)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesLibretroUriSourceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_libretro_uri_source_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GamesLibretroUriSource), 0, (GInstanceInitFunc) games_libretro_uri_source_instance_init, NULL };
	static const GInterfaceInfo games_uri_source_info = { (GInterfaceInitFunc) games_libretro_uri_source_games_uri_source_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	games_libretro_uri_source_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "GamesLibretroUriSource", &g_define_type_info, 0);
	g_type_module_add_interface (module, games_libretro_uri_source_type_id, GAMES_TYPE_URI_SOURCE, &games_uri_source_info);
	return games_libretro_uri_source_type_id;
}

