/* desktop-tracker-uri-query.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from desktop-tracker-uri-query.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <glib-object.h>
#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "games-desktop-plugin.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY (games_desktop_tracker_uri_query_get_type ())
#define GAMES_DESKTOP_TRACKER_URI_QUERY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY, GamesDesktopTrackerUriQuery))
#define GAMES_DESKTOP_TRACKER_URI_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY, GamesDesktopTrackerUriQueryClass))
#define GAMES_IS_DESKTOP_TRACKER_URI_QUERY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY))
#define GAMES_IS_DESKTOP_TRACKER_URI_QUERY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY))
#define GAMES_DESKTOP_TRACKER_URI_QUERY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY, GamesDesktopTrackerUriQueryClass))

typedef struct _GamesDesktopTrackerUriQuery GamesDesktopTrackerUriQuery;
typedef struct _GamesDesktopTrackerUriQueryClass GamesDesktopTrackerUriQueryClass;
typedef struct _GamesDesktopTrackerUriQueryPrivate GamesDesktopTrackerUriQueryPrivate;
enum  {
	GAMES_DESKTOP_TRACKER_URI_QUERY_0_PROPERTY,
	GAMES_DESKTOP_TRACKER_URI_QUERY_NUM_PROPERTIES
};
static GParamSpec* games_desktop_tracker_uri_query_properties[GAMES_DESKTOP_TRACKER_URI_QUERY_NUM_PROPERTIES];

struct _GamesDesktopTrackerUriQuery {
	GObject parent_instance;
	GamesDesktopTrackerUriQueryPrivate * priv;
};

struct _GamesDesktopTrackerUriQueryClass {
	GObjectClass parent_class;
};

static gpointer games_desktop_tracker_uri_query_parent_class = NULL;
static GamesTrackerUriQueryIface * games_desktop_tracker_uri_query_games_tracker_uri_query_parent_iface = NULL;
static GType games_desktop_tracker_uri_query_type_id = 0;

GType games_desktop_tracker_uri_query_register_type (GTypeModule * module);
VALA_EXTERN GType games_desktop_tracker_uri_query_get_type (void) G_GNUC_CONST ;
static gchar* games_desktop_tracker_uri_query_real_get_query (GamesTrackerUriQuery* base);
VALA_EXTERN GamesDesktopTrackerUriQuery* games_desktop_tracker_uri_query_new (void);
VALA_EXTERN GamesDesktopTrackerUriQuery* games_desktop_tracker_uri_query_construct (GType object_type);

static gchar*
games_desktop_tracker_uri_query_real_get_query (GamesTrackerUriQuery* base)
{
	GamesDesktopTrackerUriQuery * self;
	gchar* _tmp0_;
	gchar* result;
	self = (GamesDesktopTrackerUriQuery*) base;
	_tmp0_ = g_strdup ("SELECT nie:url(?urn) WHERE { ?urn nie:isLogicalPartOf 'urn:software-ca" \
"tegory:Game' . }");
	result = _tmp0_;
	return result;
}

GamesDesktopTrackerUriQuery*
games_desktop_tracker_uri_query_construct (GType object_type)
{
	GamesDesktopTrackerUriQuery * self = NULL;
	self = (GamesDesktopTrackerUriQuery*) g_object_new (object_type, NULL);
	return self;
}

GamesDesktopTrackerUriQuery*
games_desktop_tracker_uri_query_new (void)
{
	return games_desktop_tracker_uri_query_construct (GAMES_TYPE_DESKTOP_TRACKER_URI_QUERY);
}

static void
games_desktop_tracker_uri_query_class_init (GamesDesktopTrackerUriQueryClass * klass,
                                            gpointer klass_data)
{
	games_desktop_tracker_uri_query_parent_class = g_type_class_peek_parent (klass);
}

static void
games_desktop_tracker_uri_query_games_tracker_uri_query_interface_init (GamesTrackerUriQueryIface * iface,
                                                                        gpointer iface_data)
{
	games_desktop_tracker_uri_query_games_tracker_uri_query_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_query = (gchar* (*) (GamesTrackerUriQuery*)) games_desktop_tracker_uri_query_real_get_query;
}

static void
games_desktop_tracker_uri_query_instance_init (GamesDesktopTrackerUriQuery * self,
                                               gpointer klass)
{
}

GType
games_desktop_tracker_uri_query_get_type (void)
{
	return games_desktop_tracker_uri_query_type_id;
}

GType
games_desktop_tracker_uri_query_register_type (GTypeModule * module)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesDesktopTrackerUriQueryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_desktop_tracker_uri_query_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GamesDesktopTrackerUriQuery), 0, (GInstanceInitFunc) games_desktop_tracker_uri_query_instance_init, NULL };
	static const GInterfaceInfo games_tracker_uri_query_info = { (GInterfaceInitFunc) games_desktop_tracker_uri_query_games_tracker_uri_query_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	games_desktop_tracker_uri_query_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "GamesDesktopTrackerUriQuery", &g_define_type_info, 0);
	g_type_module_add_interface (module, games_desktop_tracker_uri_query_type_id, GAMES_TYPE_TRACKER_URI_QUERY, &games_tracker_uri_query_info);
	return games_desktop_tracker_uri_query_type_id;
}

