/* uri-game-factory.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from uri-game-factory.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static gchar** games_uri_game_factory_real_get_mime_types (GamesUriGameFactory* self,
                                                    gint* result_length1);
static gchar** games_uri_game_factory_real_get_schemes (GamesUriGameFactory* self,
                                                 gint* result_length1);
static GType games_uri_game_factory_get_type_once (void);

static gchar**
games_uri_game_factory_real_get_mime_types (GamesUriGameFactory* self,
                                            gint* result_length1)
{
	gchar** _tmp0_;
	gchar** _tmp1_;
	gint _tmp1__length1;
	gchar** result;
	_tmp0_ = g_new0 (gchar*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

gchar**
games_uri_game_factory_get_mime_types (GamesUriGameFactory* self,
                                       gint* result_length1)
{
	GamesUriGameFactoryIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_URI_GAME_FACTORY_GET_INTERFACE (self);
	if (_iface_->get_mime_types) {
		return _iface_->get_mime_types (self, result_length1);
	}
	return NULL;
}

static gchar**
games_uri_game_factory_real_get_schemes (GamesUriGameFactory* self,
                                         gint* result_length1)
{
	gchar** _tmp0_;
	gchar** _tmp1_;
	gint _tmp1__length1;
	gchar** result;
	_tmp0_ = g_new0 (gchar*, 0 + 1);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = 0;
	if (result_length1) {
		*result_length1 = _tmp1__length1;
	}
	result = _tmp1_;
	return result;
}

gchar**
games_uri_game_factory_get_schemes (GamesUriGameFactory* self,
                                    gint* result_length1)
{
	GamesUriGameFactoryIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_URI_GAME_FACTORY_GET_INTERFACE (self);
	if (_iface_->get_schemes) {
		return _iface_->get_schemes (self, result_length1);
	}
	return NULL;
}

void
games_uri_game_factory_add_uri (GamesUriGameFactory* self,
                                GamesUri* uri)
{
	GamesUriGameFactoryIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = GAMES_URI_GAME_FACTORY_GET_INTERFACE (self);
	if (_iface_->add_uri) {
		_iface_->add_uri (self, uri);
	}
}

GamesGame*
games_uri_game_factory_query_game_for_uri (GamesUriGameFactory* self,
                                           GamesUri* uri)
{
	GamesUriGameFactoryIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GAMES_URI_GAME_FACTORY_GET_INTERFACE (self);
	if (_iface_->query_game_for_uri) {
		return _iface_->query_game_for_uri (self, uri);
	}
	return NULL;
}

void
games_uri_game_factory_foreach_game (GamesUriGameFactory* self,
                                     GamesGameCallback game_callback,
                                     gpointer game_callback_target)
{
	GamesUriGameFactoryIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = GAMES_URI_GAME_FACTORY_GET_INTERFACE (self);
	if (_iface_->foreach_game) {
		_iface_->foreach_game (self, game_callback, game_callback_target);
	}
}

void
games_uri_game_factory_set_game_added_callback (GamesUriGameFactory* self,
                                                GamesGameCallback game_callback,
                                                gpointer game_callback_target)
{
	GamesUriGameFactoryIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = GAMES_URI_GAME_FACTORY_GET_INTERFACE (self);
	if (_iface_->set_game_added_callback) {
		_iface_->set_game_added_callback (self, game_callback, game_callback_target);
	}
}

static void
games_uri_game_factory_default_init (GamesUriGameFactoryIface * iface,
                                     gpointer iface_data)
{
	iface->get_mime_types = games_uri_game_factory_real_get_mime_types;
	iface->get_schemes = games_uri_game_factory_real_get_schemes;
}

static GType
games_uri_game_factory_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesUriGameFactoryIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_uri_game_factory_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType games_uri_game_factory_type_id;
	games_uri_game_factory_type_id = g_type_register_static (G_TYPE_INTERFACE, "GamesUriGameFactory", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (games_uri_game_factory_type_id, G_TYPE_OBJECT);
	return games_uri_game_factory_type_id;
}

GType
games_uri_game_factory_get_type (void)
{
	static volatile gsize games_uri_game_factory_type_id__once = 0;
	if (g_once_init_enter (&games_uri_game_factory_type_id__once)) {
		GType games_uri_game_factory_type_id;
		games_uri_game_factory_type_id = games_uri_game_factory_get_type_once ();
		g_once_init_leave (&games_uri_game_factory_type_id__once, games_uri_game_factory_type_id);
	}
	return games_uri_game_factory_type_id__once;
}

