/* cue-sheet-error.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from cue-sheet-error.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include "gnome-games.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GAMES_TYPE_CUE_SHEET_ERROR (games_cue_sheet_error_get_type ())

typedef enum  {
	GAMES_CUE_SHEET_ERROR_UNEXPECTED_TOKEN,
	GAMES_CUE_SHEET_ERROR_UNEXPECTED_EOL,
	GAMES_CUE_SHEET_ERROR_UNEXPECTED_EOF,
	GAMES_CUE_SHEET_ERROR_INVALID_FILE_FORMAT,
	GAMES_CUE_SHEET_ERROR_INVALID_TRACK_NUMBER,
	GAMES_CUE_SHEET_ERROR_INVALID_TRACK_MODE,
	GAMES_CUE_SHEET_ERROR_NOT_A_TRACK
} GamesCueSheetError;
#define GAMES_CUE_SHEET_ERROR games_cue_sheet_error_quark ()

VALA_EXTERN GQuark games_cue_sheet_error_quark (void);
VALA_EXTERN GType games_cue_sheet_error_get_type (void) G_GNUC_CONST ;
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

GQuark
games_cue_sheet_error_quark (void)
{
	return g_quark_from_static_string ("games-cue-sheet-error-quark");
}

static GType
games_cue_sheet_error_get_type_once (void)
{
	static const GEnumValue values[] = {{GAMES_CUE_SHEET_ERROR_UNEXPECTED_TOKEN, "GAMES_CUE_SHEET_ERROR_UNEXPECTED_TOKEN", "unexpected-token"}, {GAMES_CUE_SHEET_ERROR_UNEXPECTED_EOL, "GAMES_CUE_SHEET_ERROR_UNEXPECTED_EOL", "unexpected-eol"}, {GAMES_CUE_SHEET_ERROR_UNEXPECTED_EOF, "GAMES_CUE_SHEET_ERROR_UNEXPECTED_EOF", "unexpected-eof"}, {GAMES_CUE_SHEET_ERROR_INVALID_FILE_FORMAT, "GAMES_CUE_SHEET_ERROR_INVALID_FILE_FORMAT", "invalid-file-format"}, {GAMES_CUE_SHEET_ERROR_INVALID_TRACK_NUMBER, "GAMES_CUE_SHEET_ERROR_INVALID_TRACK_NUMBER", "invalid-track-number"}, {GAMES_CUE_SHEET_ERROR_INVALID_TRACK_MODE, "GAMES_CUE_SHEET_ERROR_INVALID_TRACK_MODE", "invalid-track-mode"}, {GAMES_CUE_SHEET_ERROR_NOT_A_TRACK, "GAMES_CUE_SHEET_ERROR_NOT_A_TRACK", "not-a-track"}, {0, NULL, NULL}};
	GType games_cue_sheet_error_type_id;
	games_cue_sheet_error_type_id = g_enum_register_static ("GamesCueSheetError", values);
	return games_cue_sheet_error_type_id;
}

GType
games_cue_sheet_error_get_type (void)
{
	static volatile gsize games_cue_sheet_error_type_id__once = 0;
	if (g_once_init_enter (&games_cue_sheet_error_type_id__once)) {
		GType games_cue_sheet_error_type_id;
		games_cue_sheet_error_type_id = games_cue_sheet_error_get_type_once ();
		g_once_init_leave (&games_cue_sheet_error_type_id__once, games_cue_sheet_error_type_id);
	}
	return games_cue_sheet_error_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

