/* input-capabilities.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from input-capabilities.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType games_input_capabilities_get_type_once (void);

gboolean
games_input_capabilities_get_allow_keyboard_mode (GamesInputCapabilities* self)
{
	GamesInputCapabilitiesIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_INPUT_CAPABILITIES_GET_INTERFACE (self);
	if (_iface_->get_allow_keyboard_mode) {
		return _iface_->get_allow_keyboard_mode (self);
	}
	return FALSE;
}

gboolean
games_input_capabilities_get_allow_gamepad_mode (GamesInputCapabilities* self)
{
	GamesInputCapabilitiesIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_INPUT_CAPABILITIES_GET_INTERFACE (self);
	if (_iface_->get_allow_gamepad_mode) {
		return _iface_->get_allow_gamepad_mode (self);
	}
	return FALSE;
}

static void
games_input_capabilities_default_init (GamesInputCapabilitiesIface * iface,
                                       gpointer iface_data)
{
}

static GType
games_input_capabilities_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesInputCapabilitiesIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_input_capabilities_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType games_input_capabilities_type_id;
	games_input_capabilities_type_id = g_type_register_static (G_TYPE_INTERFACE, "GamesInputCapabilities", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (games_input_capabilities_type_id, G_TYPE_OBJECT);
	return games_input_capabilities_type_id;
}

GType
games_input_capabilities_get_type (void)
{
	static volatile gsize games_input_capabilities_type_id__once = 0;
	if (g_once_init_enter (&games_input_capabilities_type_id__once)) {
		GType games_input_capabilities_type_id;
		games_input_capabilities_type_id = games_input_capabilities_get_type_once ();
		g_once_init_leave (&games_input_capabilities_type_id__once, games_input_capabilities_type_id);
	}
	return games_input_capabilities_type_id__once;
}

