/* firmware-error.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from firmware-error.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

GQuark
games_firmware_error_quark (void)
{
	return g_quark_from_static_string ("games-firmware-error-quark");
}

static GType
games_firmware_error_get_type_once (void)
{
	static const GEnumValue values[] = {{GAMES_FIRMWARE_ERROR_FIRMWARE_NOT_FOUND, "GAMES_FIRMWARE_ERROR_FIRMWARE_NOT_FOUND", "firmware-not-found"}, {0, NULL, NULL}};
	GType games_firmware_error_type_id;
	games_firmware_error_type_id = g_enum_register_static ("GamesFirmwareError", values);
	return games_firmware_error_type_id;
}

GType
games_firmware_error_get_type (void)
{
	static volatile gsize games_firmware_error_type_id__once = 0;
	if (g_once_init_enter (&games_firmware_error_type_id__once)) {
		GType games_firmware_error_type_id;
		games_firmware_error_type_id = games_firmware_error_get_type_once ();
		g_once_init_leave (&games_firmware_error_type_id__once, games_firmware_error_type_id);
	}
	return games_firmware_error_type_id__once;
}

