/* header-bar-widget.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from header-bar-widget.vala, do not modify */

/* This file is part of GNOME Games. License: GPL-3.0+.*/

#include "gnome-games.h"
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType games_header_bar_widget_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

gboolean
games_header_bar_widget_get_block_autohide (GamesHeaderBarWidget* self)
{
	GamesHeaderBarWidgetIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = GAMES_HEADER_BAR_WIDGET_GET_INTERFACE (self);
	if (_iface_->get_block_autohide) {
		return _iface_->get_block_autohide (self);
	}
	return FALSE;
}

static void
games_header_bar_widget_default_init (GamesHeaderBarWidgetIface * iface,
                                      gpointer iface_data)
{
	g_object_interface_install_property (iface, g_param_spec_boolean ("block-autohide", "block-autohide", "block-autohide", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
games_header_bar_widget_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GamesHeaderBarWidgetIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) games_header_bar_widget_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType games_header_bar_widget_type_id;
	games_header_bar_widget_type_id = g_type_register_static (G_TYPE_INTERFACE, "GamesHeaderBarWidget", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (games_header_bar_widget_type_id, gtk_widget_get_type ());
	return games_header_bar_widget_type_id;
}

GType
games_header_bar_widget_get_type (void)
{
	static volatile gsize games_header_bar_widget_type_id__once = 0;
	if (g_once_init_enter (&games_header_bar_widget_type_id__once)) {
		GType games_header_bar_widget_type_id;
		games_header_bar_widget_type_id = games_header_bar_widget_get_type_once ();
		g_once_init_leave (&games_header_bar_widget_type_id__once, games_header_bar_widget_type_id);
	}
	return games_header_bar_widget_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

